/*
 * Copyright (c) 2012, Brandon Falk <bfalk@brandonfa.lk>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 	1. Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 *
 * 	2. Redistributions in binary form must reproduce the above copyright
 * 	notice, this list of conditions and the following disclaimer in the
 * 	documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "ulpm.h"

/* getdh()
 *
 * Summary:
 *
 * This function returns the driver handle.
 *
 * Cleanup:
 *
 * Call CloseHandle() on the returned handle when it is no longer needed.
 *
 * Returns:
 *
 * INVALID_HANDLE_VALUE on error - GetLastError() for more info
 * Anything else on sucess
 */
HANDLE
getdh(void)
{
	return CreateFileW(
			L"\\\\.\\ulpm",
			GENERIC_READ | GENERIC_WRITE,
			0,
			NULL,
			OPEN_EXISTING,
			FILE_FLAG_OVERLAPPED,
			NULL);
}

/* iomem()
 *
 * Summary:
 *
 * This function performs the requested I/O of the memory.
 *
 * Params:
 *
 * HANDLE         dh    - Driver handle from getdh()
 * unsigned char *ptr   - Buffer to be written or read to. Assumed to be valid
 *                        for size bytes.
 * uint64_t       loc   - Physical memory address
 * uint32_t       size  - Amount of memory to read/write
 * uint32_t       write - Either PM_READ to read, or PM_WRITE to write.
 *
 * Returns:
 *
 *  0 on success
 * -1 on user-mode error, GetLastError() for more info
 *  Anything else is a custom error code, see the PMERR_xxx defines in api.h
 */
int
iomem(HANDLE dh, unsigned char *ptr, uint64_t loc, uint32_t size,
		uint32_t write)
{
	uint32_t       recvd;
	struct _reqmem req;

	req.loc   = loc;
	req.size  = size;
	req.write = write;

	if(!DeviceIoControl(dh,
				        IOCTL_REQ_MEM,
						&req,
						sizeof(req),
						ptr,
						size,
						(DWORD*)&recvd,
						NULL)){
		return -1;
	}

	return recvd;
}

