/*
 * Copyright (c) 2012, Brandon Falk <bfalk@brandonfa.lk>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 	1. Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 *
 * 	2. Redistributions in binary form must reproduce the above copyright
 * 	notice, this list of conditions and the following disclaimer in the
 * 	documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "ulpm.h"

/* main()
 *
 * Summary:
 *
 * This demonstrates use of the API by dumping the bootsector at 0x7C00,
 * given it was not overwritten.
 *
 * Returns:
 *
 *  0 on success
 * -1 on error
 */
int
main(void)
{
	HANDLE dh;	
	int    sts, i;

	unsigned char buf[512], *ptr;

	/* Get driver handle */
	dh = getdh();
	if(dh == INVALID_HANDLE_VALUE){
		fprintf(stderr, "getdh() error : %d\n", GetLastError());
		return -1;
	}

	/* Dump memory */
	sts = iomem(dh, buf, 0x7C00, 512, PM_READ);
	if(sts == -1){
		CloseHandle(dh);
		fprintf(stderr, "iomem(user) error : %d\n", GetLastError());
		return -1;
	} else if(sts) {
		CloseHandle(dh);
		fprintf(stderr, "iomem(kern) error : %d\n", sts);
		return -1;
	}

	/* Print the hex */
	i   = 512;
	ptr = buf;
	while(i--){
		printf("%.2x ", *ptr);

		if(!(i % 16))
			putchar('\n');
		ptr++;
	}

	puts("\n\n-------------------------------------------------\n");
	ZeroMemory(buf, 512);

	/* Write memory */
	sts = iomem(dh, buf, 0x7C00, 512, PM_WRITE);
	if(sts == -1){
		CloseHandle(dh);
		fprintf(stderr, "iomem(user) error : %d\n", GetLastError());
		return -1;
	} else if(sts) {
		CloseHandle(dh);
		fprintf(stderr, "iomem(kern) error : %d\n", sts);
		return -1;
	}

	/* Dump memory */
	sts = iomem(dh, buf, 0x7C00, 512, PM_READ);
	if(sts == -1){
		CloseHandle(dh);
		fprintf(stderr, "iomem(user) error : %d\n", GetLastError());
		return -1;
	} else if(sts) {
		CloseHandle(dh);
		fprintf(stderr, "iomem(kern) error : %d\n", sts);
		return -1;
	}

	/* Print the hex */
	i   = 512;
	ptr = buf;
	while(i--){
		printf("%.2x ", *ptr);

		if(!(i % 16))
			putchar('\n');
		ptr++;
	}

	CloseHandle(dh);

	return 0;
}

