/*
 * Copyright (c) 2012, Brandon Falk <bfalk@brandonfa.lk>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 	1. Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 *
 * 	2. Redistributions in binary form must reproduce the above copyright
 * 	notice, this list of conditions and the following disclaimer in the
 * 	documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _ULPM_H_
#define _ULPM_H_

#pragma warning(push, 0)
#include <wdm.h>
#pragma warning(pop)

#define RELEASE 0 /* 0 for debug, 1 for release */

/* Make a globally switchable DbgPrint, to shut the driver up for release
 * builds. Use dbgprint() instead of DbgPrint() everywhere in this driver
 * to allow to turn everything off without painful find and replace.
 */
#if     RELEASE == 0
#define dbgprint(x, ...) DbgPrint(x, __VA_ARGS__)
#else
#define dbgprint(x, ...)
#endif

#if RELEASE == 0

char irplookup[28][32] =
{
	"IRP_MJ_CREATE",
	"IRP_MJ_CREATE_NAMED_PIPE",
	"IRP_MJ_CLOSE",
	"IRP_MJ_READ",
	"IRP_MJ_WRITE",
	"IRP_MJ_QUERY_INFORMATION",
	"IRP_MJ_SET_INFORMATION",
	"IRP_MJ_QUERY_EA",
	"IRP_MJ_SET_EA",
	"IRP_MJ_FLUSH_BUFFERS",
	"IRP_MJ_QUERY_VOLUME_INFORMATION",
	"IRP_MJ_SET_VOLUME_INFORMATION",
	"IRP_MJ_DIRECTORY_CONTROL",
	"IRP_MJ_FILE_SYSTEM_CONTROL",
	"IRP_MJ_DEVICE_CONTROL",
	"IRP_MJ_INTERNAL_DEVICE_CONTROL",
	"IRP_MJ_SHUTDOWN",
	"IRP_MJ_LOCK_CONTROL",
	"IRP_MJ_CLEANUP",
	"IRP_MJ_CREATE_MAILSLOT",
	"IRP_MJ_QUERY_SECURITY",
	"IRP_MJ_SET_SECURITY",
	"IRP_MJ_POWER",
	"IRP_MJ_SYSTEM_CONTROL",
	"IRP_MJ_DEVICE_CHANGE",
	"IRP_MJ_QUERY_QUOTA",
	"IRP_MJ_SET_QUOTA",
	"IRP_MJ_PNP"
};

#define IRP_LOOKUP(x) ((x >= 0 && x <= 0x1b) ? irplookup[x] : "Unknown")
#else
#define IRP_LOOKUP(x) NULL
#endif

#define IOCTL_REQ_MEM \
	CTL_CODE(FILE_DEVICE_DATALINK, 0x1337, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* Error with the input to the driver */
#define PMERR_INPUT    1
/* Error with the output pointed given to the driver */
#define PMERR_OUTPUT   2
/* Internal error, when the driver can not get the physical memory */
#define PMERR_INTERNAL 3
/* Bounds error */
#define PMERR_BOUNDS   4

#pragma pack(push, 1)
struct _reqmem {
	unsigned __int64 loc;
	unsigned __int32 size;
	unsigned __int32 write;
};
#pragma pack(pop)

DRIVER_INITIALIZE
DriverEntry;

DRIVER_UNLOAD
drvunload;

__drv_dispatchType(IRP_MJ_CREATE) /* I know... but OACR complains */
__drv_dispatchType(IRP_MJ_CREATE_NAMED_PIPE)
__drv_dispatchType(IRP_MJ_CLOSE)
__drv_dispatchType(IRP_MJ_READ)
__drv_dispatchType(IRP_MJ_WRITE)
__drv_dispatchType(IRP_MJ_QUERY_INFORMATION)
__drv_dispatchType(IRP_MJ_SET_INFORMATION)
__drv_dispatchType(IRP_MJ_QUERY_EA)
__drv_dispatchType(IRP_MJ_SET_EA)
__drv_dispatchType(IRP_MJ_FLUSH_BUFFERS)
__drv_dispatchType(IRP_MJ_QUERY_VOLUME_INFORMATION)
__drv_dispatchType(IRP_MJ_SET_VOLUME_INFORMATION)
__drv_dispatchType(IRP_MJ_DIRECTORY_CONTROL)
__drv_dispatchType(IRP_MJ_FILE_SYSTEM_CONTROL)
__drv_dispatchType(IRP_MJ_INTERNAL_DEVICE_CONTROL)
__drv_dispatchType(IRP_MJ_SHUTDOWN)
__drv_dispatchType(IRP_MJ_LOCK_CONTROL)
__drv_dispatchType(IRP_MJ_CLEANUP)
__drv_dispatchType(IRP_MJ_CREATE_MAILSLOT)
__drv_dispatchType(IRP_MJ_QUERY_SECURITY)
__drv_dispatchType(IRP_MJ_SET_SECURITY)
__drv_dispatchType(IRP_MJ_POWER)
__drv_dispatchType(IRP_MJ_SYSTEM_CONTROL)
__drv_dispatchType(IRP_MJ_DEVICE_CHANGE)
__drv_dispatchType(IRP_MJ_QUERY_QUOTA)
__drv_dispatchType(IRP_MJ_SET_QUOTA)
__drv_dispatchType(IRP_MJ_PNP)
DRIVER_DISPATCH
genhdlr;

__drv_dispatchType(IRP_MJ_CREATE)
DRIVER_DISPATCH
create;

__drv_dispatchType(IRP_MJ_DEVICE_CONTROL)
DRIVER_DISPATCH
devctrl;

#endif

